[Mesa-dev] [PATCH] glsl: error out if non-constant indexing of SSBO arrays with GLSL ES

Tapani Pälli tapani.palli at intel.com
Thu Oct 1 04:54:33 PDT 2015


Fixes Piglit compiler test:
	shader-storage-block-array-dynamic-indexing.frag

Signed-off-by: Tapani Pälli <tapani.palli at intel.com>
---
 src/glsl/ast_array_index.cpp | 14 ++++++++------
 1 file changed, 8 insertions(+), 6 deletions(-)

diff --git a/src/glsl/ast_array_index.cpp b/src/glsl/ast_array_index.cpp
index dfb3107..5e8f49d 100644
--- a/src/glsl/ast_array_index.cpp
+++ b/src/glsl/ast_array_index.cpp
@@ -231,15 +231,17 @@ _mesa_ast_array_index_to_hir(void *mem_ctx,
             _mesa_glsl_error(&loc, state, "unsized array index must be constant");
          }
       } else if (array->type->fields.array->is_interface()
-                 && array->variable_referenced()->data.mode == ir_var_uniform
+                 && (array->variable_referenced()->data.mode == ir_var_uniform ||
+                     array->variable_referenced()->data.mode == ir_var_shader_storage)
                  && !state->is_version(400, 0) && !state->ARB_gpu_shader5_enable) {
-	 /* Page 46 in section 4.3.7 of the OpenGL ES 3.00 spec says:
+	 /* Page 50 in section 4.3.9 of the OpenGL ES 3.10 spec says:
 	  *
 	  *     "All indexes used to index a uniform block array must be
 	  *     constant integral expressions."
 	  */
-	 _mesa_glsl_error(&loc, state,
-			  "uniform block array index must be constant");
+	 _mesa_glsl_error(&loc, state, "%s block array index must be constant",
+                          array->variable_referenced()->data.mode
+                          == ir_var_uniform ? "uniform" : "shader storage");
       } else {
 	 /* whole_variable_referenced can return NULL if the array is a
 	  * member of a structure.  In this case it is safe to not update
@@ -231,15 +231,17 @@ _mesa_ast_array_index_to_hir(void *mem_ctx,
 	  *
-	  *     "All indexes used to index a uniform block array must be
-	  *     constant integral expressions."
+	  *     "All indices used to index a uniform or shader storage block
+	  *     array must be constant integral expressions."
 	  */
-- 
2.4.3



More information about the mesa-dev mailing list